open Match_unif
open Projet


(* Pretty printing *)

(* Du pretty printing très général pour les listes *)

let string_of_list str l delim =
  match l with
    | [] -> "[]"
    | x::q ->
        "["^
          (str x)^
          (List.fold_left (fun acc s -> acc^delim^(str s)) "" q)^
          "]";;


(* Pour les termes *)
(* Les variables sont affichées x, y, z... *)
(* Pour certains symboles de fonctions, on a un affichage plus "user
   friendly": par exemple, Term("/",[Var 0;Var 1]) est affiché x/y. *)

let rec string_of_term subst t =
  match t with
    | Var v ->
        (match subst v with
           | Some s -> s
           | None -> "Var "^(string_of_int v))
    | Term(s,l) when s = "+" -> (
        match l with
          | [a;b] -> "("^(string_of_term subst a)^" + "^(string_of_term subst b)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "-" -> (
        match l with
          | [a;b] -> "("^(string_of_term subst a)^" - "^(string_of_term subst b)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "." -> (
        match l with
          | [a;b] -> "("^(string_of_term subst a)^" . "^(string_of_term subst b)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "/" -> (
        match l with
          | [a;b] -> "("^(string_of_term subst a)^" / "^(string_of_term subst b)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = ":" -> (
        match l with
          | [a;b] -> "("^(string_of_term subst a)^" : "^(string_of_term subst b)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "or" -> (
        match l with
          | [a;b] -> "("^(string_of_term subst a)^" or "^(string_of_term subst b)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "and" -> (
        match l with
          | [a;b] -> "("^(string_of_term subst a)^" and "^(string_of_term subst b)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "pi" -> (
        match l with
          | [a;b] -> "pi("^(string_of_term subst a)^","^(string_of_term subst b)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "S" -> (
        match l with
          | [a] -> "(S "^(string_of_term subst a)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "I" -> (
        match l with
          | [a] -> "I("^(string_of_term subst a)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "int" -> (
        match l with
          | [a;b] -> "int("^(string_of_term subst a)^","^(string_of_term subst b)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "intlist" -> (
        match l with
          | [a] -> "intlist("^(string_of_term subst a)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "f" -> (
        match l with
          | [a] -> "f("^(string_of_term subst a)^")"
          | [a;b;c] -> "f("^(string_of_term subst a)^","^(string_of_term subst b)^","^(string_of_term subst c)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "not" -> (
        match l with
          | [a] -> "not("^(string_of_term subst a)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "a" -> (
        match l with
          | [a] -> "a("^(string_of_term subst a)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "b" -> (
        match l with
          | [a] -> "b("^(string_of_term subst a)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "c" -> (
        match l with
          | [a] -> "c("^(string_of_term subst a)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "#" -> (
        match l with
          | [a] -> "#("^(string_of_term subst a)^")"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "0" -> (
        match l with
          | [] -> "0"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "1" -> (
        match l with
          | [] -> "1"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "e" -> (
        match l with
          | [] -> "e"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) when s = "[]" -> (
        match l with
          | [] -> "[]"
          | _ -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")"
      )
    | Term(s,l) -> "Term("^s^","^(string_of_list (fun x -> x) (List.map (string_of_term subst) l) ";")^")";;


(* Pour les paires de dépendance *)

let string_of_dp subst dp =
  let (l,r) = dp in
  "("^(string_of_term subst l)^","^(string_of_term subst r)^")";;


(* Pour les symboles de fonction *)

let string_of_symb s = s;;


(* Pour les symboles de fonction avec leur arité *)

let string_of_symb_arit (s,n) = "("^s^","^(string_of_int n)^")";;


(* Pour l'"ôtabilité" des paires de dépendance *)

let string_of_can_be_removed rem =
  match rem with
    | Strict -> "Strict"
    | Large -> "Large"
    | No -> "No";;


(* Pour les projections simples *)

let string_of_proj p =
  string_of_list (fun (s,n) -> "("^s^","^(string_of_int n)^")") p "; ";;
