open Match_unif


type var = int;;
type symb = string;;
type def_symbs = symb list;;

(* les types principaux *)
type rule = term * term;; (* l -> r est reprsente par (l,r) *)
type system = rule list;; (* les regles sont supposes regulires, avec membre  gauche non variable *)
type dp = term * term;;
type graph = {graph : Graph.Pack.Digraph.t; vertex_of_dp : (dp, Graph.Pack.Digraph.V.t) Hashtbl.t; dp_of_vertex : (Graph.Pack.Digraph.V.t, dp) Hashtbl.t};;

(* compute_def_symbs calcule l'ensemble des symboles dfinis du systme donn en entre *)
val compute_def_symbs : system -> def_symbs;;

(* compute_def_symbs_arit calcule l'ensemble des symboles dfinis avec leur arit du systme donn en entre *)
val compute_def_symbs_arit : system -> (symb*int) list;;

(* compute_def_symbs calcule l'ensemble des symboles du systme donn en entre *)
val compute_symbs : system -> symb list;;

(* compute_def_symbs calcule l'ensemble des symboles avec leur arit du systme donn en entre *)
val compute_symbs_arit : system -> (symb*int) list;;

(* compute_def_symbs calcule l'ensemble des symboles constructeurs du systme donn en entre *)
val compute_const_symbs : system -> symb list;;

(* compute_dps calcule toutes les paires de dpendance du systme donn en entre *)
val compute_dps : system -> dp list;;

(* (compute_graph const_symbs dp_list) calcule une sur-approximation G_init du graphe de dpendance
pour la liste de paires de dpendance dp_list, en supposant que const_symbs est la liste
des symboles contructeurs,
suivant la mthode du cours (cfr dfinition 7.14 p. 81 *)
val compute_graph : symb list -> dp list -> graph;;

(* (extract_components G) retourne la liste des composantes fortement connexes de son entre G *)
val extract_components : graph -> graph list;;

(* (compute_n_step_reds R n t) retourne la liste des rduits en n pas du terme t par le systme R *)
val compute_n_step_reds : system -> int -> term -> term list;;

(* (project P t) retourne t si t est une variable,
                          ti tel que ti est le (P f)-ieme element de l si t = (Term f l) *)
val project : (string*int) list -> term -> term

type can_be_removed = Strict | Large | No;;

(* removable R P (u,v) retourne
 - Strict si (P u) (|> U ->R)+ (P v)
 - Large si (P u) (|> U ->R)* (P v)
 - No dans les autres cas *)
val removable : system -> (string*int) list -> dp -> can_be_removed


(* find_projection R G Def n retourne une projection P sous forme d'une
   liste d'association entre les symboles dfinis donns par def_symbs
   et leur projection, et une paire de dpendance (u0,v0) telle que
   removable R P (u0,v0) = Strict, et pour toute paire (u,v) dans la
   composante fortement connexe de (u0,v0) dans G, removable R P (u,v) =
   Strict ou removable R P (u,v) = Large, si aucune telle projection
   n'existe, on peut lever une exception. *)
val find_projection : system -> graph -> (symb * int) list -> (symb * int) list * dp

(* main R retourne une liste de sous-graphes Gi du graphe Ginit dont
la terminaison est quivalente  celle de Ginit, et qui sont minimaux au sens
o le thorme de l'enonc ne permet plus de les rduire. *)
val main : system -> graph list
